local MAJOR_VERSION = "Threat-2.0"
local MINOR_VERSION = tonumber(("$Revision: 68842 $"):match("%d+"))

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then _G.ThreatLib_MINOR_VERSION = MINOR_VERSION end

local _, c = _G.UnitClass("player")
if c ~= "PALADIN" then return end 

local GetTime = _G.GetTime

ThreatLib_funcs[#ThreatLib_funcs+1] = function()

	local ThreatLib = _G.ThreatLib
	local Paladin = ThreatLib:GetOrCreateModule("Player")

	local _G = _G
	local ipairs = _G.ipairs
	local select = _G.select
	local GetTalentInfo = _G.GetTalentInfo

	local irfRanks = {0, 0.16, 0.33, 0.5}

	local HolyHealIDs = {
		-- Holy Light
		635, 639, 647, 1026, 1042, 3472, 10328, 10329, 25292, 27135, 27136,
		
		-- Holy Shock
		20473, 20929, 20930, 27174, 33072,
		
		-- Lay on Hands
		633, 2800, 10310, 27154,
		
		-- Seal of Light
		20165, 20347, 20348, 20349, 27160,
		
		-- Flash of Light
		19750, 19939, 19940, 19941, 19942, 19943, 27137
	}

	local rfOn = false
	local fanaticismModifier = 0
	local RIGHTEOUS_FURY_SPELL_ID = 25780
	
	function Paladin:ClassInit()
		self.schoolThreatMods[_G.SCHOOL_MASK_HOLY] = self.RighteousFury
		
		-- Righteous Fury
		self.BuffHandlers[RIGHTEOUS_FURY_SPELL_ID] = self.RighteousFuryBuff
		
		local healMod = function(self, amt, isDamage)
			local dMod = 1
			if isDamage then
				dMod = 2
			end
			return self:RighteousFury(amt) * 0.5 * dMod
		end

		for i = 1, #HolyHealIDs do
			self.AbilityHandlers[HolyHealIDs[i]] = healMod
		end
		
		-- Judgement
		-- This is a maximum of like 10 TPS, do we really need it?
		-- We'll track first-lands, but not worry about refreshes.
		self.CastLandedHandlers[20271] = function(self, spellID, recipient)
			self:AddTargetThreat(recipient, self:RighteousFury(57))
		end
		
		local holyShield = function(self, amt, isDamage)
			return amt * 1.35
		end
		local holyShieldIDs = {20925, 20927, 20928, 27179}
		for i = 1, #holyShieldIDs do
			self.AbilityHandlers[holyShieldIDs[i]] = holyShield
		end
		
		-- Righteous Defense
		-- The player spell is 31789
		-- The taunt effect is 31790
		self.CastLandedHandlers[31789] = self.CastRighteousDefense
		self.MobDebuffHandlers[31790] = self.RighteousDefense
		-- Mob debuff handler here!
		
		HolyHealIDs = nil
		self.RighteousDefenseCastTime = 0
	end
	
	function Paladin:ClassEnable()
		self.passiveThreatModifiers = 1 - fanaticismModifier
	end

	function Paladin:ScanTalents()
		-- Scan talents	
		self.righteousFuryMod = 1.6 + (0.6 * irfRanks[select(5, GetTalentInfo(2, 7))+1])
		
		-- Fanaticism
		local rank = select(5, GetTalentInfo(3, 21))
		fanaticismModifier = (0.06 * rank)
		self:calcBuffMods()
	end

	function Paladin:RighteousFuryBuff(action, spellID, applications)
		self.isTanking = false
		if action == "gain" or action == "exist" then
			rfOn = true
			self.isTanking = true
			self.passiveThreatModifiers = 1
		elseif action == "lose" then
			rfOn = false
			self.passiveThreatModifiers = 1 - fanaticismModifier
		end
		if action == "gain" or action == "lose" then
			self.totalThreatMods = nil
		end
		ThreatLib:Debug("passiveThreatModifiers is %s", self.passiveThreatModifiers)
	end

	function Paladin:RighteousFury(amt)
		if rfOn then
			return amt * self.righteousFuryMod
		else 
			return amt
		end
	end

	function Paladin:CastRighteousDefense(spellID, target)
		self.RighteousDefenseCastTime = GetTime()
	end
	
	function Paladin:RighteousDefense(spellID, target)
		if GetTime() - self.RighteousDefenseCastTime < 1 then
			ThreatLib:Debug("Got Righteous Defense (%s) on target %s, max threat on target is %s", spellID, target, (ThreatLib:GetMaxThreatOnTarget(target)))
			self:SetTargetThreat(target, ThreatLib:GetMaxThreatOnTarget(target))
		end
	end
end
